#include <iostream>
#include <vector>
#include <string>
#include <sstream>
#include "histogram.h"
//#include "text.h"
#include "svg.h"
#include <curl/curl.h>

using namespace std;

struct Input {
    vector<double> numbers;
    vector<string> colors;
    size_t bin_count{};
};

Input
input_data(istream& in_stream,bool prompt) {
    size_t number_count;
    Input in;

    if (prompt) cerr << "Enter number count: ";
    in_stream >> number_count;
    vector<double> numbers(number_count);

    in.numbers.resize(number_count);

    for (size_t i = 0; i < number_count; i++) {
        in_stream >> in.numbers[i];
    }

    if (prompt) cerr << "Enter bins count and colors: ";
    in_stream >> in.bin_count;
    vector<string> colors(in.bin_count);
    in.colors.resize(in.bin_count);

    for (size_t i = 0; i < in.bin_count; i++) {
        //in_stream >> in.colors[i];
        in.colors[i]="red";
        bool check = check_color(in.colors[i]);
        while (!check) {
        if (prompt) cerr << "Incorrect input. Color doesn't contain # or has spaces." << endl;
        in_stream >> in.colors[i];
        //in.colors[i]="red";
        check = check_color(in.colors[i]);
        }
    }
    return in;
}

size_t write_data(void* items, size_t item_size, size_t item_count, void* ctx) {
    size_t data_size = item_size * item_count;
    stringstream* buffer = reinterpret_cast<stringstream*>(ctx);
    (*buffer).write(reinterpret_cast<const char*>(items), data_size);
    return data_size;
}

size_t write_header(char* buffer, size_t size, size_t nitems, void* userdata)
{
    return size * nitems;
}

Input download(const string& address) {
    curl_global_init(CURL_GLOBAL_ALL);
    stringstream buffer;

    CURL* curl = curl_easy_init();
    if (curl) {
        CURLcode res;
        curl_easy_setopt(curl, CURLOPT_URL, address.c_str());
        curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, write_data);
        curl_easy_setopt(curl, CURLOPT_WRITEDATA, &buffer);
        curl_easy_setopt(curl, CURLOPT_HEADERFUNCTION, write_header);
        curl_easy_setopt(curl, CURLOPT_HEADERDATA, NULL);
        res = curl_easy_perform(curl);
        curl_easy_cleanup(curl);

        if (res != CURLE_OK) {
            cout << curl_easy_strerror(res);
            exit(1);
        }
        else
        {
            double *ct;
            res = curl_easy_getinfo(curl, CURLINFO_CONNECT_TIME, &ct);
            if((CURLE_OK == res) && ct)
            printf("Time: %f\n",ct);
        }
    }

    return input_data(buffer, false);
}

int main(int argc, char* argv[])
{
    Input in;
    if (argc > 1)
    {
        in = download(argv[1]);
    }
    else
    {
        in = input_data(cin,true);
    }
    vector<size_t> bins = make_histogram(in.numbers, in.bin_count);
    //show_histogram_text(bins, in.bin_count);
    show_histogram_svg(bins, in.colors);
}
